"""
App init
"""

# Django
from django.utils.translation import gettext_lazy as _

__version__ = "2.12.1"
__title__ = _("Ship Replacement")

__package_name__ = "aa-srp"
__app_name__ = "aasrp"
__app_name_verbose__ = "AA SRP"
__app_name_useragent__ = "AA-SRP"

__github_url__ = f"https://github.com/ppfeufer/{__package_name__}"



# Generated by Django 4.0.10 on 2023-03-06 09:09

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aasrp", "0008_aasrprequestcomment_new_status_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="FleetType",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "name",
                    models.CharField(
                        help_text="Descriptive name of your fleet type", max_length=254
                    ),
                ),
                (
                    "is_enabled",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Whether this fleet type is active or not",
                    ),
                ),
            ],
            options={
                "verbose_name": "SRP Link Fleet Type",
                "verbose_name_plural": "SRP Link Fleet Types",
                "default_permissions": (),
            },
        ),
        migrations.AddField(
            model_name="aasrplink",
            name="fleet_type",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="The SRP link fleet type, if it's set",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="aasrp.fleettype",
            ),
        ),
    ]



"""
Notifications helper
"""

# Django
from django.conf import settings
from django.contrib.auth.models import User
from django.template.loader import render_to_string
from django.urls import reverse

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.discord.channel_message import send_message_to_discord_channel
from aasrp.discord.direct_message import send_user_notification
from aasrp.models import Setting, SrpRequest

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


def notify_requester(
    requester: User,
    reviser: User,
    srp_request: SrpRequest,
    comment: str,
    message_level: str = "success",
) -> None:
    """
    Send SRP request notification to the requester

    :param requester: The user who made the SRP request
    :type requester: User
    :param reviser: The user who is revising the SRP request
    :type reviser: User
    :param srp_request: The SRP request object
    :type srp_request: SrpRequest
    :param comment: The comment made by the reviser
    :type comment: str
    :param message_level: The level of the message (success, error, info, etc. Default: success)
    :type message_level: str
    :return: None
    :rtype: None
    """

    request_status = srp_request.get_request_status_display()

    context = {
        "ship": srp_request.ship.name,
        "srp_name": srp_request.srp_link.srp_name,
        "status": request_status.lower(),
        "srp_code": srp_request.srp_link.srp_code,
        "request_code": srp_request.request_code,
        "reviser": reviser,
        "comment": comment,
    }

    allianceauth_notification = render_to_string(
        template_name="aasrp/notifications/allianceauth/request-status-change.html",
        context=context,
    )

    discord_notification = render_to_string(
        template_name="aasrp/notifications/discord/request-status-change.html",
        context=context,
    )

    send_user_notification(
        user=requester,
        level=message_level,
        title=f"SRP Request {request_status}",
        message={
            "allianceauth": allianceauth_notification,
            "discord": discord_notification,
        },
    )


def notify_srp_team(srp_request: SrpRequest, additional_info: str) -> None:
    """
    Send SRP request notification to the SRP teams Discord channel

    :param srp_request: The SRP request object
    :type srp_request: SrpRequest
    :param additional_info: Additional information to be included in the notification
    :type additional_info: str
    :return: None
    :rtype: None
    """

    srp_team_discord_channel = Setting.objects.get_setting(
        setting_key=Setting.Field.SRP_TEAM_DISCORD_CHANNEL_ID
    )

    if srp_team_discord_channel:
        site_base_url = settings.SITE_URL
        srp_code = srp_request.srp_link.srp_code
        srp_link = site_base_url + reverse(
            viewname="aasrp:view_srp_requests", args=[srp_code]
        )

        message = render_to_string(
            template_name="aasrp/notifications/discord/srp-team.html",
            context={
                "request_code": srp_request.request_code,
                "character": srp_request.character.character_name,
                "ship": srp_request.ship.name,
                "killboard_link": srp_request.killboard_link,
                "additional_info": additional_info.replace("@", "{@}"),
                "srp_code": srp_code,
                "srp_link": srp_link,
            },
        )

        logger.info(
            msg="Sending SRP request notification to the SRP team channel on Discord"
        )

        send_message_to_discord_channel(
            channel_id=srp_team_discord_channel,
            title="New SRP Request",
            message=message,
        )



"""
Handling Discord direct messages to a user
"""

# pylint: disable=import-outside-toplevel, duplicate-code

# Standard Library
from datetime import datetime

# Django
from django.contrib.auth.models import User
from django.utils import timezone

# Alliance Auth
from allianceauth.notifications import notify
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.app_settings import (
    DISCORDPROXY_HOST,
    DISCORDPROXY_PORT,
    DISCORDPROXY_TIMEOUT,
    aa_discordnotify_installed,
    allianceauth_discordbot_installed,
    discordproxy_installed,
)
from aasrp.constants import DISCORD_EMBED_COLOR_MAP

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def _aadiscordbot_send_private_message(
    user_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
) -> None:
    """
    Try to send a PM to a user on Discord via allianceauth-discordbot

    :param user_id:
    :type user_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    if allianceauth_discordbot_installed():
        logger.debug(
            msg="allianceauth-discordbot is active, trying to send private message"
        )

        # Third Party
        from aadiscordbot.tasks import send_message
        from discord import Embed

        embed = Embed(
            title=str(title),
            description=message,
            color=DISCORD_EMBED_COLOR_MAP.get(level),
            timestamp=datetime.now(),
        )

        if embed_message is True:
            send_message(user_id=user_id, embed=embed)
        else:
            send_message(user_id=user_id, message=f"**{title}**\n\n{message}")
    else:
        logger.debug(
            msg=(
                "allianceauth-discordbot is not available on this "
                "system to send the private message"
            )
        )


def _discordproxy_send_private_message(
    user_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
):
    """
    Try to send a PM to a user on Discord via discordproxy
    (fall back to allianceauth-discordbot if needed)

    :param user_id:
    :type user_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    # Third Party
    from discordproxy.client import DiscordClient
    from discordproxy.exceptions import DiscordProxyException

    target = f"{DISCORDPROXY_HOST}:{DISCORDPROXY_PORT}"
    client = DiscordClient(target=target, timeout=DISCORDPROXY_TIMEOUT)

    try:
        logger.debug(msg="Trying to send a direct message via discordproxy")

        if embed_message is True:
            # Third Party
            from discordproxy.discord_api_pb2 import Embed

            footer = Embed.Footer(text=str(__title__))
            embed = Embed(
                title=str(title),
                description=message,
                color=DISCORD_EMBED_COLOR_MAP.get(level),
                timestamp=timezone.now().isoformat(),
                footer=footer,
            )

            client.create_direct_message(user_id=user_id, embed=embed)
        else:
            client.create_direct_message(
                user_id=user_id, content=f"**{title}**\n\n{message}"
            )
    except DiscordProxyException as ex:
        # Something went wrong with discordproxy
        # Fail silently and try if allianceauth-discordbot is available
        # as a last ditch effort to get the message out to Discord
        logger.debug(
            msg=(
                "Something went wrong with discordproxy, "
                "cannot send a direct message, trying allianceauth-discordbot "
                f"to send the message if available. Error: {ex}"
            )
        )

        _aadiscordbot_send_private_message(
            user_id=user_id,
            level=level,
            title=title,
            message=message,
            embed_message=embed_message,
        )


def send_user_notification(
    user: User,
    title: str,
    message: dict[str, str],
    embed_message: bool = True,
    level: str = "info",
) -> None:
    """
    Send notification to user
    This creates a notification in Auth and a PM in Discord when either
    Discordproxy, AA-Discordbot or AA Discord Notifications is installed

    :param user:
    :type user:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    if message["allianceauth"]:
        getattr(notify, level)(user=user, title=title, message=message["allianceauth"])

    # Handle Discord PMs when aa_discordnotify is not active
    # Check if either allianceauth_discordbot or discordproxy are available
    # to send the PM
    if hasattr(user, "discord"):  # Check if the user has a Discord account
        logger.debug(msg="User has a Discord account")

        # Check if discordnotify is active
        if not aa_discordnotify_installed():
            if discordproxy_installed():
                logger.debug(msg="discordproxy seems to be available ...")

                _discordproxy_send_private_message(
                    user_id=int(user.discord.uid),
                    level=level,
                    title=title,
                    message=message["discord"],
                    embed_message=embed_message,
                )
            else:
                # discordproxy not available, try if allianceauth-discordbot is
                # available
                logger.debug(
                    msg=(
                        "discordproxy not available to send a direct message, "
                        "let's see if we can use allianceauth-discordbot if available"
                    )
                )

                _aadiscordbot_send_private_message(
                    user_id=int(user.discord.uid),
                    level=level,
                    title=title,
                    message=message["discord"],
                    embed_message=embed_message,
                )
        else:
            logger.debug(
                msg="discordnotify is active, we don't have to send the PM ourself."
            )
    else:
        logger.debug(
            msg="User doesn't have a Discord account, can't send any messages ..."
        )



# Django
from django.test import TestCase

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth (External Libs)
from eveuniverse.core.eveimageserver import character_portrait_url, type_render_url

# AA SRP
from aasrp.helper.eve_images import (
    get_character_portrait_from_evecharacter,
    get_type_render_url_from_type_id,
)


class TestGetTypeRenderUrlFromTypeId(TestCase):
    """
    Test the get_type_render_url_from_type_id function.
    """

    def test_type_render_url_as_html_with_name(self):
        """
        Test the get_type_render_url_from_type_id function with HTML output and name.

        :return:
        :rtype:
        """

        evetype_id = 123
        size = 64
        evetype_name = "Test Type"
        result = get_type_render_url_from_type_id(
            evetype_id, size, evetype_name, as_html=True
        )
        self.assertIn('<img class="aasrp-evetype-icon rounded"', result)
        self.assertIn(f'src="{type_render_url(type_id=evetype_id, size=size)}"', result)
        self.assertIn(f'alt="{evetype_name}"', result)
        self.assertIn('loading="lazy"', result)

    def test_type_render_url_as_html_without_name(self):
        """
        Test the get_type_render_url_from_type_id function with HTML output without name.

        :return:
        :rtype:
        """

        evetype_id = 123
        size = 64
        result = get_type_render_url_from_type_id(evetype_id, size, as_html=True)

        self.assertIn('<img class="aasrp-evetype-icon rounded"', result)
        self.assertIn(f'src="{type_render_url(type_id=evetype_id, size=size)}"', result)
        self.assertNotIn('alt="', result)
        self.assertIn('loading="lazy"', result)

    def test_type_render_url_as_plain_url(self):
        """
        Test the get_type_render_url_from_type_id function with plain URL output.

        :return:
        :rtype:
        """

        evetype_id = 123
        size = 64
        result = get_type_render_url_from_type_id(evetype_id, size, as_html=False)

        self.assertEqual(result, type_render_url(type_id=evetype_id, size=size))

    def test_type_render_url_with_default_size(self):
        """
        Test the get_type_render_url_from_type_id function with default size.

        :return:
        :rtype:
        """

        evetype_id = 123
        result = get_type_render_url_from_type_id(evetype_id)

        self.assertEqual(result, type_render_url(type_id=evetype_id, size=32))


class TestGetCharacterPortraitFromEvecharacter(TestCase):
    """
    Test the get_character_portrait_from_evecharacter function.
    """

    def test_character_portrait_as_html_with_name(self):
        """
        Test the get_character_portrait_from_evecharacter function with HTML output and name.

        :return:
        :rtype:
        """

        character = EveCharacter.objects.create(
            character_id=12345,
            character_name="Test Character",
            corporation_id=2001,
            corporation_name="Test Corp",
        )
        result = get_character_portrait_from_evecharacter(
            character, size=64, as_html=True
        )

        self.assertIn('<img class="aasrp-character-portrait rounded"', result)
        self.assertIn(
            f'src="{character_portrait_url(character_id=12345, size=64)}"', result
        )
        self.assertIn('alt="Test Character"', result)
        self.assertIn('loading="lazy"', result)

    def test_character_portrait_as_plain_url(self):
        """
        Test the get_character_portrait_from_evecharacter function with plain URL output.

        :return:
        :rtype:
        """

        character = EveCharacter.objects.create(
            character_id=12345,
            character_name="Test Character",
            corporation_id=2001,
            corporation_name="Test Corp",
        )
        result = get_character_portrait_from_evecharacter(
            character, size=64, as_html=False
        )

        self.assertEqual(result, character_portrait_url(character_id=12345, size=64))

    def test_character_portrait_with_default_size(self):
        """
        Test the get_character_portrait_from_evecharacter function with default size.

        :return:
        :rtype:
        """

        character = EveCharacter.objects.create(
            character_id=12345,
            character_name="Test Character",
            corporation_id=2001,
            corporation_name="Test Corp",
        )
        result = get_character_portrait_from_evecharacter(character)

        self.assertEqual(result, character_portrait_url(character_id=12345, size=32))



"""
Helper functions for SRP data
"""

# Django
from django.utils.translation import gettext_lazy as _

# Alliance Auth (External Libs)
from eveuniverse.models import EveType

# AA SRP
from aasrp.helper.icons import copy_to_clipboard_icon
from aasrp.helper.numbers import l10n_number_format
from aasrp.models import SrpRequest


def request_code_html(request_code: str) -> str:
    """
    Get HTML for request code with copy to clipboard icon

    :param request:
    :type request:
    :param request_code:
    :type request_code:
    :return:
    :rtype:
    """

    ctc_icon = copy_to_clipboard_icon(
        data=request_code, title=_("Copy request code to clipboard")
    )

    return f"{request_code}<sup>{ctc_icon}</sup>"


def payout_amount_html(payout_amount: int) -> str:
    """
    Get HTML for payout amount with copy to clipboard icon

    :param request:
    :type request:
    :param payout_amount:
    :type payout_amount:
    :return:
    :rtype:
    """

    payout_amount_ctc_icon = copy_to_clipboard_icon(
        data=str(payout_amount), title=_("Copy payout amount to clipboard")
    )
    payout_amount_localized = l10n_number_format(payout_amount)

    return (
        '<span class="srp-payout d-flex justify-content-end align-items-baseline">'
        '<span class="srp-payout-tooltip"><span class="srp-payout-amount d-block cursor-pointer">'
        f"{payout_amount_localized} ISK</span></span><sup>{payout_amount_ctc_icon}</sup></span>"
    )


def localized_isk_value(loss_amount: int) -> str:
    """
    Get localized ISK value with currency symbol

    :param loss_amount:
    :type loss_amount:
    :return:
    :rtype:
    """

    return f"{l10n_number_format(loss_amount)} ISK"


def request_fleet_details_html(srp_request: SrpRequest) -> str:
    """
    Get HTML for fleet details

    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    l10n_srp_code = _("SRP code")
    l10n_request_code = _("Request code")

    fleet_name = f"<p>{srp_request.srp_link.srp_name}</p>"
    fleet_details = (
        '<p class="small text-muted">'
        f"{l10n_srp_code}: {srp_request.srp_link.srp_code}<br>"
        f"{l10n_request_code}: {srp_request.request_code}</p>"
    )

    return f"{fleet_name}{fleet_details}"


def attempt_to_re_add_ship_information_to_request(
    srp_request: SrpRequest,
) -> SrpRequest:
    """
    If for some reason the ship gets removed from EveType table,
    srp_request.ship is None. In this case, we have to re-add the ship to prevent
    errors in our DataTables …

    :param srp_request:
    :type srp_request:
    :return:
    :rtype:
    """

    srp_kill_link_id = SrpRequest.objects.get_kill_id(
        killboard_link=srp_request.killboard_link
    )

    (
        ship_type_id,
        ship_value,  # pylint: disable=unused-variable
        victim_id,  # pylint: disable=unused-variable
    ) = SrpRequest.objects.get_kill_data(kill_id=srp_kill_link_id)
    (
        srp_request__ship,
        created_from_esi,  # pylint: disable=unused-variable
    ) = EveType.objects.get_or_create_esi(id=ship_type_id)

    srp_request.ship_name = srp_request__ship.name
    srp_request.ship = srp_request__ship
    srp_request.save()

    return srp_request



# Standard Library
from datetime import datetime
from http import HTTPStatus
from unittest.mock import patch

# Django
from django.contrib import messages
from django.contrib.auth.models import Permission, User
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse

# AA SRP
from aasrp.models import SrpLink
from aasrp.tests.utils import get_permission_content_type
from aasrp.views.general import (
    complete_srp_link,
    delete_srp_link,
    disable_srp_link,
    enable_srp_link,
    request_srp,
    srp_link_view_requests,
)


class TestSrpLinkDelete(TestCase):
    """
    Test the srp_link_delete view.
    """

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_srp_link_deleted_successfully(self, mock_get_srp_link):
        """
        Test the srp_link_delete view for a user with the manage_srp permissions and successful deletion.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(srp_code="SRP123", fleet_time=datetime.now())
        mock_get_srp_link.return_value = srp_link

        factory = RequestFactory()
        request = factory.get(reverse("aasrp:delete_srp_link", args=["SRP123"]))
        request.user = user

        # Add session and messages middleware to the request
        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = delete_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_srp_link_not_found(self, mock_get_srp_link):
        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get_srp_link.side_effect = SrpLink.DoesNotExist

        factory = RequestFactory()
        request = factory.get(reverse("aasrp:delete_srp_link", args=["SRP123"]))
        request.user = user

        # Add session and messages middleware to the request
        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = delete_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)


class TestSrpLinkDisable(TestCase):
    """
    Test the srp_link_disable view.
    """

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_disable_srp_link_successfully(self, mock_get_srp_link):
        """
        Test the srp_link_disable view for a user with the manage_srp permissions and successful disabling.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(srp_code="SRP123", fleet_time=datetime.now())
        mock_get_srp_link.return_value = srp_link

        factory = RequestFactory()
        request = factory.post(reverse("aasrp:disable_srp_link", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = disable_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_disable_srp_link_not_found(self, mock_get_srp_link):
        """
        Test the srp_link_disable view for a user with the manage_srp permissions and not found.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get_srp_link.side_effect = SrpLink.DoesNotExist

        factory = RequestFactory()
        request = factory.post(reverse("aasrp:disable_srp_link", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = disable_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)


class TestSrpLinkEnable(TestCase):
    """
    Test the srp_link_enable view.
    """

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_enable_srp_link_successfully(self, mock_get_srp_link):
        """
        Test the srp_link_enable view for a user with the manage_srp permissions and successful enabling.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(
            srp_code="SRP123",
            srp_status=SrpLink.Status.CLOSED,
            fleet_time=datetime.now(),
        )
        mock_get_srp_link.return_value = srp_link

        factory = RequestFactory()
        request = factory.get(reverse("aasrp:enable_srp_link", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = enable_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)
        srp_link.refresh_from_db()
        self.assertEqual(srp_link.srp_status, SrpLink.Status.ACTIVE)

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_enable_srp_link_not_found(self, mock_get_srp_link):
        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get_srp_link.side_effect = SrpLink.DoesNotExist

        factory = RequestFactory()
        request = factory.post(reverse("aasrp:enable_srp_link", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = enable_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)


class TestSrpLinkViewRequests(TestCase):
    """
    Test the srp_link_view_requests view.
    """

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_view_requests_srp_link_not_found(self, mock_get_srp_link):
        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get_srp_link.side_effect = SrpLink.DoesNotExist

        factory = RequestFactory()
        request = factory.get(reverse("aasrp:view_srp_requests", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = srp_link_view_requests(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)
        self.assertEqual(len(messages.get_messages(request)), 1)
        self.assertEqual(
            str(list(messages.get_messages(request))[0]),
            "Unable to locate SRP link with ID SRP123",
        )


class TestCompleteSrpLink(TestCase):
    """
    Test the complete_srp_link view.
    """

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_complete_srp_link_successfully(self, mock_get_srp_link):
        """
        Test the complete_srp_link view for a user with the manage_srp permissions and successful completion.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(
            srp_code="SRP123",
            srp_status=SrpLink.Status.ACTIVE,
            fleet_time=datetime.now(),
        )
        mock_get_srp_link.return_value = srp_link

        factory = RequestFactory()
        request = factory.get(reverse("aasrp:complete_srp_link", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = complete_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)
        self.assertEqual(len(messages.get_messages(request)), 1)
        self.assertEqual(
            str(list(messages.get_messages(request))[0]), "SRP link marked as completed"
        )
        srp_link.refresh_from_db()
        self.assertEqual(srp_link.srp_status, SrpLink.Status.COMPLETED)

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_complete_srp_link_not_found(self, mock_get_srp_link):
        """
        Test the complete_srp_link view for a user with the manage_srp permissions and not found.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get_srp_link.side_effect = SrpLink.DoesNotExist

        factory = RequestFactory()
        request = factory.get(reverse("aasrp:complete_srp_link", args=["SRP123"]))
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = complete_srp_link(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)
        self.assertEqual(len(messages.get_messages(request)), 1)
        self.assertEqual(
            str(list(messages.get_messages(request))[0]),
            "Unable to locate SRP link with ID SRP123",
        )


class TestRequestSrp(TestCase):
    """
    Test the request_srp view.
    """

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_request_srp_link_not_found(self, mock_get_srp_link):
        """
        Test the request_srp view for a user with the basic_access permissions and not found.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="basic_access"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        mock_get_srp_link.side_effect = SrpLink.DoesNotExist

        factory = RequestFactory()
        request = factory.post(
            reverse("aasrp:request_srp", args=["SRP123"]),
            {
                "killboard_link": "https://zkillboard.com/kill/12345/",
                "additional_info": "Test info",
            },
        )
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = request_srp(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)
        self.assertEqual(len(messages.get_messages(request)), 1)
        self.assertEqual(
            str(list(messages.get_messages(request))[0]),
            "Unable to locate SRP Fleet using SRP code SRP123",
        )

    @patch("aasrp.views.general.SrpLink.objects.get")
    def test_request_srp_link_not_active(self, mock_get_srp_link):
        """
        Test the request_srp view for a user with the basic_access permissions and not active.

        :param mock_get_srp_link:
        :type mock_get_srp_link:
        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="basic_access"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)
        srp_link = SrpLink.objects.create(
            srp_code="SRP123",
            srp_status=SrpLink.Status.CLOSED,
            fleet_time=datetime.now(),
        )
        mock_get_srp_link.return_value = srp_link

        factory = RequestFactory()
        request = factory.post(
            reverse("aasrp:request_srp", args=["SRP123"]),
            {
                "killboard_link": "https://zkillboard.com/kill/12345/",
                "additional_info": "Test info",
            },
        )
        request.user = user

        session_middleware = SessionMiddleware(lambda req: None)
        session_middleware.process_request(request)
        request.session.save()

        message_middleware = MessageMiddleware(lambda req: None)
        message_middleware.process_request(request)

        response = request_srp(request, "SRP123")

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertIn(reverse("aasrp:srp_links"), response.url)
        self.assertEqual(len(messages.get_messages(request)), 1)
        self.assertEqual(
            str(list(messages.get_messages(request))[0]),
            "This SRP link is no longer available for SRP requests.",
        )



"""
Versioned static URLs to break browser caches when changing the app version
"""

# Django
from django.contrib.auth.models import User
from django.template.defaulttags import register

# Alliance Auth
from allianceauth.framework.api.user import get_main_character_name_from_user
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


@register.filter
def main_character_name(user: User) -> str:
    """
    Get the users main character name, or return the username if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    return get_main_character_name_from_user(user=user)


@register.filter
def main_character_id(user: User) -> int:
    """
    Get the users main character id, or return 1 if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user and hasattr(user.profile.main_character, "character_id"):
        return user.profile.main_character.character_id

    return 1


@register.filter
def main_character_corporation_name(user: User) -> str:
    """
    Get the users main character corporation name, or an empty string if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user and hasattr(user.profile.main_character, "corporation_name"):
        return user.profile.main_character.corporation_name

    return ""


@register.filter
def main_character_corporation_id(user: User) -> int:
    """
    Get the users main character corporation id, or 1 if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user and hasattr(user.profile.main_character, "corporation_id"):
        return user.profile.main_character.corporation_id

    return 1


@register.filter
def main_character_alliance_name(user: User) -> str:
    """
    Get the users main character alliance name, or an empty string if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    return getattr(user.profile.main_character, "alliance_name", "") if user else ""


@register.filter
def main_character_alliance_id(user: User) -> int:
    """
    Get the users main character alliance id, or 1 if no main character

    :param user:
    :type user:
    :return:
    :rtype:
    """

    if user is None:
        return 1

    try:
        return_value = int(user.profile.main_character.alliance_id)
    except (AttributeError, ValueError, TypeError):
        return_value = 1

    return return_value



"""
Test cases for the helper functions in the aasrp.helper.srp_data module.
"""

# Django
from django.test import TestCase, override_settings

# AA SRP
from aasrp.helper.icons import copy_to_clipboard_icon
from aasrp.helper.srp_data import (
    localized_isk_value,
    payout_amount_html,
    request_code_html,
    request_fleet_details_html,
)
from aasrp.models import SrpLink, SrpRequest


class TestPayoutAmountHtml(TestCase):
    """
    Test cases for the payout_amount_html function.
    """

    @override_settings(LANGUAGE_CODE="en")
    def test_payout_amount_html_returns_correct_html_locale_en(self):
        """
        Test localization of payout amount HTML in English locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(1000)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">1,000 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="1000"', result)

    @override_settings(LANGUAGE_CODE="de")
    def test_payout_amount_html_returns_correct_html_locale_de(self):
        """
        Test localization of payout amount HTML in German locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(1000)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">1.000 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="1000"', result)

    @override_settings(LANGUAGE_CODE="en")
    def test_payout_amount_html_handles_zero_locale_en(self):
        """
        Test localization of payout amount HTML with zero in English locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(0)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">0 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="0"', result)

    @override_settings(LANGUAGE_CODE="de")
    def test_payout_amount_html_handles_zero_locale_de(self):
        """
        Test localization of payout amount HTML with zero in German locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(0)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">0 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="0"', result)

    @override_settings(LANGUAGE_CODE="en")
    def test_payout_amount_html_handles_negative_amount_locale_en(self):
        """
        Test localization of payout amount HTML with negative amount in English locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(-1000)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">-1,000 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="-1000"', result)

    @override_settings(LANGUAGE_CODE="de")
    def test_payout_amount_html_handles_negative_amount_locale_de(self):
        """
        Test localization of payout amount HTML with negative amount in German locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(-1000)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">-1.000 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="-1000"', result)

    @override_settings(LANGUAGE_CODE="en")
    def test_payout_amount_html_handles_large_amount_locale_en(self):
        """
        Test localization of payout amount HTML with a large amount in English locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(1000000000)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">1,000,000,000 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="1000000000"', result)

    @override_settings(LANGUAGE_CODE="de")
    def test_payout_amount_html_handles_large_amount_locale_de(self):
        """
        Test localization of payout amount HTML with a large amount in German locale.

        :return:
        :rtype:
        """

        result = payout_amount_html(1000000000)

        self.assertIn(
            '<span class="srp-payout-amount d-block cursor-pointer">1.000.000.000 ISK</span>',
            result,
        )
        self.assertIn('data-clipboard-text="1000000000"', result)


class TestRequestCodeHtml(TestCase):
    """
    Test cases for the request_code_html function.
    """

    def test_request_code_html_returns_correct_html(self):
        """
        Test returning correct HTML for request code.

        :return:
        :rtype:
        """

        result = request_code_html("ABC123")

        icon = copy_to_clipboard_icon(
            data="ABC123", title="Copy request code to clipboard"
        )

        self.assertEqual(result, f"ABC123<sup>{icon}</sup>")


class TestLocalizedIskValue(TestCase):
    """
    Test cases for the localized_isk_value function.
    """

    @override_settings(LANGUAGE_CODE="en")
    def test_localized_isk_value_returns_correct_html_locale_en(self):
        """
        Test localization of ISK value in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(1000), "1,000 ISK")

    @override_settings(LANGUAGE_CODE="de")
    def test_localized_isk_value_returns_correct_html_locale_de(self):
        """
        Test localization of ISK value in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(1000), "1.000 ISK")

    @override_settings(LANGUAGE_CODE="en")
    def test_localized_isk_value_handles_zero_locale_en(self):
        """
        Test localization of ISK value with zero in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(0), "0 ISK")

    @override_settings(LANGUAGE_CODE="de")
    def test_localized_isk_value_handles_zero_locale_de(self):
        """
        Test localization of ISK value with zero in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(0), "0 ISK")

    @override_settings(LANGUAGE_CODE="en")
    def test_localized_isk_value_handles_negative_amount_locale_en(self):
        """
        Test localization of ISK value with negative amount in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(-1000), "-1,000 ISK")

    @override_settings(LANGUAGE_CODE="de")
    def test_localized_isk_value_handles_negative_amount_locale_de(self):
        """
        Test localization of ISK value with negative amount in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(-1000), "-1.000 ISK")

    @override_settings(LANGUAGE_CODE="en")
    def test_localized_isk_value_handles_large_amount_locale_en(self):
        """
        Test localization of ISK value with a large amount in English locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(1000000000), "1,000,000,000 ISK")

    @override_settings(LANGUAGE_CODE="de")
    def test_localized_isk_value_handles_large_amount_locale_de(self):
        """
        Test localization of ISK value with a large amount in German locale.

        :return:
        :rtype:
        """

        self.assertEqual(localized_isk_value(1000000000), "1.000.000.000 ISK")


class TestRequestFleetDetailsHtml(TestCase):
    """
    Test cases for the request_fleet_details_html function.
    """

    def test_request_fleet_details_html_returns_correct_html(self):
        """
        Test returning correct HTML for fleet details.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="Fleet Alpha", srp_code="FA123")
        srp_request = SrpRequest(srp_link=srp_link, request_code="REQ123")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p>Fleet Alpha</p>", result)
        self.assertIn("SRP code: FA123", result)
        self.assertIn("Request code: REQ123", result)

    def test_request_fleet_details_html_handles_empty_srp_name(self):
        """
        Test returning correct HTML for fleet details with empty SRP name.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="", srp_code="FA123")
        srp_request = SrpRequest(srp_link=srp_link, request_code="REQ123")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p></p>", result)
        self.assertIn("SRP code: FA123", result)
        self.assertIn("Request code: REQ123", result)

    def test_request_fleet_details_html_handles_empty_request_code(self):
        """
        Test returning correct HTML for fleet details with empty request code.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="Fleet Alpha", srp_code="FA123")
        srp_request = SrpRequest(srp_link=srp_link, request_code="")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p>Fleet Alpha</p>", result)
        self.assertIn("SRP code: FA123", result)
        self.assertIn("Request code: ", result)

    def test_request_fleet_details_html_handles_special_characters(self):
        """
        Test returning correct HTML for fleet details with special characters.

        :return:
        :rtype:
        """

        srp_link = SrpLink(srp_name="Fleet @lpha", srp_code="FA!23")
        srp_request = SrpRequest(srp_link=srp_link, request_code="REQ!23")
        result = request_fleet_details_html(srp_request)

        self.assertIn("<p>Fleet @lpha</p>", result)
        self.assertIn("SRP code: FA!23", result)
        self.assertIn("Request code: REQ!23", result)



"""
Handling Discord channel messages
"""

# pylint: disable=import-outside-toplevel, duplicate-code

# Standard Library
from datetime import datetime

# Django
from django.utils import timezone

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.app_settings import (
    DISCORDPROXY_HOST,
    DISCORDPROXY_PORT,
    DISCORDPROXY_TIMEOUT,
    allianceauth_discordbot_installed,
    discordproxy_installed,
)
from aasrp.constants import DISCORD_EMBED_COLOR_MAP

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def _aadiscordbot_send_channel_message(
    channel_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
) -> None:
    """
    Try to send a message to a channel on Discord via allianceauth-discordbot

    :param channel_id:
    :type channel_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    if allianceauth_discordbot_installed():
        logger.debug(
            msg="allianceauth-discordbot is active, trying to send channel message"
        )

        # Third Party
        from aadiscordbot.tasks import send_message
        from discord import Embed

        embed = Embed(
            title=str(title),
            description=message,
            color=DISCORD_EMBED_COLOR_MAP.get(level),
            timestamp=datetime.now(),
        )

        if embed_message is True:
            send_message(channel_id=channel_id, embed=embed)
        else:
            send_message(channel_id=channel_id, message=f"**{title}**\n\n{message}")
    else:
        logger.debug(
            msg=(
                "allianceauth-discordbot is not available on this "
                "system to send the channel message"
            )
        )


def _discordproxy_send_channel_message(
    channel_id: int,
    title: str,
    message: str,
    embed_message: bool = True,
    level: str = "info",
):
    """
    Try to send a message to a channel on Discord via discordproxy
    (fall back to allianceauth-discordbot if needed)

    :param channel_id:
    :type channel_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :param level:
    :type level:
    :return:
    :rtype:
    """

    # Third Party
    from discordproxy.client import DiscordClient
    from discordproxy.exceptions import DiscordProxyException

    target = f"{DISCORDPROXY_HOST}:{DISCORDPROXY_PORT}"
    client = DiscordClient(target=target, timeout=DISCORDPROXY_TIMEOUT)

    try:
        logger.debug(msg="Trying to send a channel message via discordproxy")

        if embed_message:
            # Third Party
            from discordproxy.discord_api_pb2 import Embed

            footer = Embed.Footer(text=str(__title__))

            embed = Embed(
                title=title,
                description=message,
                color=DISCORD_EMBED_COLOR_MAP.get(level),
                timestamp=timezone.now().isoformat(),
                footer=footer,
            )
            client.create_channel_message(channel_id=channel_id, embed=embed)
        else:
            client.create_channel_message(
                channel_id=channel_id, content=f"**{title}**\n\n{message}"
            )
    except DiscordProxyException as ex:
        # Something went wrong with discordproxy
        # Fail silently and try if allianceauth-discordbot is available
        # as a last ditch effort to get the message out to Discord
        logger.debug(
            msg=(
                "Something went wrong with discordproxy, "
                "cannot send a channel message, "
                f"trying allianceauth-discordbot to send the message. Error: {ex}"
            )
        )

        _aadiscordbot_send_channel_message(
            channel_id=channel_id,
            level=level,
            title=title,
            message=message,
            embed_message=True,
        )


def send_message_to_discord_channel(
    channel_id: int, title: str, message: str, embed_message: bool = True
) -> None:
    """
    Sending a message to a discord channel.
    This creates a message to the SRP Team channel on Discord when either
    Discordproxy or AA-Discordbot is installed

    :param channel_id:
    :type channel_id:
    :param title:
    :type title:
    :param message:
    :type message:
    :param embed_message:
    :type embed_message:
    :return:
    :rtype:
    """

    # Check if either allianceauth_discordbot or discordproxy are available
    # to send the channel message
    if discordproxy_installed():
        logger.debug(msg="discordproxy seems to be available, check if we can use it")

        _discordproxy_send_channel_message(
            channel_id=channel_id,
            level="info",
            title=title,
            message=message,
            embed_message=embed_message,
        )
    else:
        # discordproxy not available, try if allianceauth-discordbot is available
        logger.debug(
            msg=(
                "discordproxy not available to send the channel message, "
                "let's see if we can use allianceauth-discordbot"
            )
        )

        _aadiscordbot_send_channel_message(
            channel_id=channel_id,
            level="info",
            title=title,
            message=message,
            embed_message=embed_message,
        )
